@@ -10,6 +10,9 @@ SEP_SYMBOLS = frozenset(('(', ')', ',', ':'))
 # name, lineno, col_offset, end_lineno, end_col_offset
 Token = Tuple[str, int, int, int, int]
 
+COMMENT = '# no-walrus'
+SIMPLE_NODE = (ast.Name, ast.Constant)
+
 
 def name_lineno_coloffset_list(
     tokens: list[Token],
@@ -61,15 +64,36 @@ def find_names(
     return names
 
 
+def is_simple_test(node: ast.AST) -> bool:
+    # TODO: this is definitely covered, if
+    # I run tests and put a breakpoint here then it
+    # gets here, and returns both True and False.
+    # So why is it reported as uncovered?
+    return (  # pragma: no cover
+        isinstance(node, SIMPLE_NODE)
+        or (
+            isinstance(node, ast.Compare)
+            and isinstance(node.left, SIMPLE_NODE)
+            and (
+                all(
+                    isinstance(_node, SIMPLE_NODE)
+                    for _node in node.comparators
+                )
+            )
+        )
+    )
+
+
 def visit_function_def(
-    node: ast.FunctionDef,
+    node: ast.FunctionDef | ast.Module,
     path: str,
 ) -> list[tuple[Token, Token]]:
     names = set()
     assignments = set()
     ifs = set()
     for _node in ast.walk(node):
-        names.update(find_names(_node))
+        if isinstance(_node, ast.Name):
+            names.add(record_name_lineno_coloffset(_node))
 
     related_vars = {}
 
@@ -86,13 +110,15 @@ def visit_function_def(
                     ),
                 )
                 related_vars[target.id] = list(find_names(_node.value))
-        elif isinstance(_node, ast.If):
+        elif isinstance(_node, ast.If) and is_simple_test(_node.test):
             ifs.update(find_names(_node.test))
             for __node in _node.orelse:
-                if isinstance(__node, ast.If):
-                    ifs.update(find_names(__node.test))
-        elif isinstance(_node, (ast.If, ast.While)):
-            ifs.update(find_names(_node.test))
+                if isinstance(__node, ast.If) and is_simple_test(__node.test):
+                    ifs.update(
+                        find_names(
+                            __node.test,
+                        ),
+                    )
 
     sorted_names = sorted(names, key=lambda x: (x[1], x[2]))
     sorted_assignments = sorted(assignments, key=lambda x: (x[1], x[2]))
@@ -161,6 +187,7 @@ def auto_walrus(content: str, path: str, line_length: int) -> str | None:
         return None
 
     walruses = []
+    walruses.extend(visit_function_def(tree, path))
     for node in ast.walk(tree):
         if isinstance(node, ast.FunctionDef):
             walruses.extend(visit_function_def(node, path))
@@ -195,6 +222,12 @@ def auto_walrus(content: str, path: str, line_length: int) -> str | None:
             f'{lines[_assignment[1]-1][:_assignment[2]]}'
             f'{lines[_assignment[1]-1][_assignment[4]:]}'
         )
+        if (
+            COMMENT in lines[_assignment[1]-1]
+        ) or (
+            COMMENT in lines[_if_statement[1]-1]
+        ):
+            continue
         lines[_assignment[1] - 1] = line_without_assignment
         # add walrus
         lines[_if_statement[1]-1] = line_with_walrus
